package com.youdao.animation;

import android.view.View;
import android.view.animation.Interpolator;

import com.youdao.animation.nineoldandroids.animation.Animator.AnimatorListener;
import com.youdao.animation.nineoldandroids.animation.AnimatorSet;
import com.youdao.animation.nineoldandroids.view.ViewHelper;

/**
 * base view animation, which is a abstract class. if you want to use the
 * animation, you must extends the class, and implement the "prepare" method.
 * 
 * @author yuym
 * @since 2014-03-03
 */
public abstract class BaseViewAnimator {

	/** animation set */
	private AnimatorSet mAnimatorSet;

	/** animation duration */
	private long mDuration = DEFAULT_DURATION;

	/** defualt animation duration is 1000ms */
	public static final long DEFAULT_DURATION = 1000;

	/** the constructor code block for "mAnimatorSet */
	{
		mAnimatorSet = new AnimatorSet();
	}

	/**
	 * the view prepare before starting
	 * 
	 * @param target
	 *            target view
	 */
	protected abstract void prepare(View target);

	/**
	 * the view animate
	 * 
	 * @param target
	 */
	public void animate(View target) {
		reset(target);
		prepare(target);
		start();
	}

	/**
	 * reset the view to default status
	 * 
	 * @param target
	 */
	public void reset(View target) {
		ViewHelper.setAlpha(target, 1);
		ViewHelper.setScaleX(target, 1);
		ViewHelper.setScaleY(target, 1);
		ViewHelper.setTranslationX(target, 0);
		ViewHelper.setTranslationY(target, 0);
		ViewHelper.setRotation(target, 0);
		ViewHelper.setRotationY(target, 0);
		ViewHelper.setRotationX(target, 0);
		ViewHelper.setPivotX(target, target.getMeasuredWidth() / 2.0f);
		ViewHelper.setPivotY(target, target.getMeasuredHeight() / 2.0f);
	}

	/**
	 * start to animate
	 */
	public void start() {
		mAnimatorSet.setDuration(mDuration);
		mAnimatorSet.start();
	}

	/**
	 * set aimation duration
	 * 
	 * @param duration
	 * @return
	 */
	public BaseViewAnimator setDuration(long duration) {
		mDuration = duration;
		return this;
	}

	/**
	 * set delay time before the animation start
	 * 
	 * @param delay
	 * @return
	 */
	public BaseViewAnimator setStartDelay(long delay) {
		getAnimatorAgent().setStartDelay(delay);
		return this;
	}

	/**
	 * get animation delay
	 * 
	 * @return
	 */
	public long getStartDelay() {
		return mAnimatorSet.getStartDelay();
	}

	/**
	 * add animation listener
	 * 
	 * @param listener
	 * @return
	 */
	public BaseViewAnimator addAnimatorListener(AnimatorListener listener) {
		mAnimatorSet.addListener(listener);
		return this;
	}

	/**
	 * animation cancel
	 */
	public void cancel() {
		mAnimatorSet.cancel();
	}

	/**
	 * whether the animation is running
	 * 
	 * @return
	 */
	public boolean isRunning() {
		return mAnimatorSet.isRunning();
	}

	/**
	 * whether the animation is started
	 * 
	 * @return
	 */
	public boolean isStarted() {
		return mAnimatorSet.isStarted();
	}

	/**
	 * remove animation listener
	 * 
	 * @param l
	 */
	public void removeAnimatorListener(AnimatorListener listener) {
		mAnimatorSet.removeListener(listener);
	}

	/**
	 * remove all animation listeners
	 */
	public void removeAllListeners() {
		mAnimatorSet.removeAllListeners();
	}

	/**
	 * set animation interpolator
	 * 
	 * @param interpolator
	 * @return
	 */
	public BaseViewAnimator setInterpolator(Interpolator interpolator) {
		mAnimatorSet.setInterpolator(interpolator);
		return this;
	}

	/**
	 * get animation duration
	 * 
	 * @return
	 */
	public long getDuration() {
		return mDuration;
	}

	/**
	 * get animation set
	 * 
	 * @return
	 */
	public AnimatorSet getAnimatorAgent() {
		return mAnimatorSet;
	}

}
