package com.youdao.animation;

import java.util.ArrayList;
import java.util.List;

import android.view.View;
import android.view.animation.Interpolator;

import com.youdao.animation.nineoldandroids.animation.Animator;

public class YDAnimation {

	/** default animation duration */
	private static final long DURATION = BaseViewAnimator.DEFAULT_DURATION;

	/** default no delay */
	private static final long NO_DELAY = 0;

	/** base animation */
	private BaseViewAnimator animator;

	/** target view */
	private View target;

	private YDAnimation(AnimationBuilder animationComposer) {
		animator = animationComposer.animator;
		target = animationComposer.target;
	}

	/**
	 * build animation builder, if animatorClass is null, build default
	 * animation
	 * 
	 * @param animator
	 * @return
	 */
	public static AnimationBuilder builder(
			Class<? extends BaseViewAnimator> animatorClass) {
		BaseViewAnimator animator = null;
		try {
			animator = animatorClass.newInstance();
		} catch (Exception e) {
			e.printStackTrace();
		}
		if (animator == null) {
			return new AnimationBuilder(new DefaultAnimator());
		}

		return new AnimationBuilder(animator);
	}

	/**
	 * animation builder
	 * 
	 * @author yuym
	 * 
	 */
	public static final class AnimationBuilder {

		/** animation callback listeners */
		private List<Animator.AnimatorListener> callbacks = new ArrayList<Animator.AnimatorListener>();

		/** base animation */
		private BaseViewAnimator animator;

		/** animation duration */
		private long duration = DURATION;

		/** animation delay */
		private long delay = NO_DELAY;

		/** animation interpolator */
		private Interpolator interpolator;

		/** target view */
		private View target;

		private AnimationBuilder(BaseViewAnimator animator) {
			this.animator = animator;
		}

		/**
		 * set animation duration
		 * 
		 * @param duration
		 * @return
		 */
		public AnimationBuilder duration(long duration) {
			this.duration = duration;
			return this;
		}

		/**
		 * set animation delay
		 * 
		 * @param delay
		 * @return
		 */
		public AnimationBuilder delay(long delay) {
			this.delay = delay;
			return this;
		}

		/**
		 * set animation interpolator
		 * 
		 * @param interpolator
		 * @return
		 */
		public AnimationBuilder interpolate(Interpolator interpolator) {
			this.interpolator = interpolator;
			return this;
		}

		/**
		 * add Animation listener
		 * 
		 * @param listener
		 * @return
		 */
		public AnimationBuilder addAnimatorListener(
				Animator.AnimatorListener listener) {
			callbacks.add(listener);
			return this;
		}

		/**
		 * the animation of the view plays.
		 * 
		 * @return
		 */
		public YDAnimation playOn(View target) {
			this.target = target;
			animator.setDuration(duration).setInterpolator(interpolator)
					.setStartDelay(delay);
			if (callbacks.size() > 0) {
				for (Animator.AnimatorListener callback : callbacks) {
					animator.addAnimatorListener(callback);
				}
			}
			animator.animate(target);
			return new YDAnimation(this);
		}

	}

	/**
	 * whether the animation is started
	 * 
	 * @return
	 */
	public boolean isStarted() {
		return animator.isStarted();
	}

	/**
	 * whether the animation is running
	 * 
	 * @return
	 */
	public boolean isRunning() {
		return animator.isRunning();
	}

	/**
	 * stop the view animation
	 * 
	 * @param reset
	 *            whether is animation reset
	 */
	public void stop(boolean reset) {
		animator.cancel();
		if (reset)
			animator.reset(target);
	}
}
